/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ogmrip-lavc-mpeg2.h"
#include "ogmrip-plugin.h"

#include <glib/gi18n-lib.h>

static gint ogmrip_lavc_mpeg2_run (OGMJobSpawn *spawn);

G_DEFINE_TYPE (OGMRipLavcMpeg2, ogmrip_lavc_mpeg2, OGMRIP_TYPE_LAVC)

G_CONST_RETURN gchar *
ogmrip_lavc_mpeg2_get_codec (void)
{
  const gchar *codec = "mpeg2video";

  return codec;
}

static void
ogmrip_lavc_mpeg2_class_init (OGMRipLavcMpeg2Class *klass)
{
  OGMJobSpawnClass *spawn_class;
  OGMRipLavcClass *lavc_class;

  spawn_class = (OGMJobSpawnClass *) klass;
  spawn_class->run = ogmrip_lavc_mpeg2_run;

  lavc_class = (OGMRipLavcClass *) klass;
  lavc_class->get_codec = ogmrip_lavc_mpeg2_get_codec;
}

static void
ogmrip_lavc_mpeg2_init (OGMRipLavcMpeg2 *lavc_mpeg2)
{
}

static gint
ogmrip_lavc_mpeg2_run (OGMJobSpawn *spawn)
{
  ogmrip_video_codec_set_4mv (OGMRIP_VIDEO_CODEC (spawn), FALSE);

  return OGMJOB_SPAWN_CLASS (ogmrip_lavc_mpeg2_parent_class)->run (spawn);
}

static OGMRipVideoPlugin lavc_mpeg2_plugin =
{
  NULL,
  G_TYPE_NONE,
  "lavc-mpeg2",
  N_("Lavc Mpeg-2"),
  OGMRIP_FORMAT_MPEG2,
  G_MAXINT,
  8
};

OGMRipVideoPlugin *
ogmrip_init_plugin (void)
{
  gchar *output;
  gboolean match;

  if (!g_spawn_command_line_sync ("mencoder -ovc help", &output, NULL, NULL, NULL))
    return NULL;

  match = g_regex_match_simple ("^ *lavc *- .*$", output, G_REGEX_MULTILINE, 0);
  g_free (output);

  if (!match)
    return NULL;

  lavc_mpeg2_plugin.type = OGMRIP_TYPE_LAVC_MPEG2;

  return &lavc_mpeg2_plugin;
}

