/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib/gi18n.h>
#include <gtk/gtkdialog.h>
#include <glade/glade.h>

#include "ogmrip-plugin.h"
#include "ogmrip-options-plugin.h"
#include "ogmrip-preferences.h"
#include "ogmrip-helper.h"
#include "ogmrip-mpeg.h"

#define OGMRIP_GLADE_FILE "ogmrip/ogmrip-mpeg.glade"
#define OGMRIP_GLADE_ROOT "root"

#define OGMRIP_TYPE_MPEG_DIALOG          (ogmrip_mpeg_dialog_get_type ())
#define OGMRIP_MPEG_DIALOG(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), OGMRIP_TYPE_MPEG_DIALOG, OGMRipMpegDialog))
#define OGMRIP_MPEG_DIALOG_CLASS(klass)  (G_TYPE_CHECK_CLASS_CAST ((klass), OGMRIP_TYPE_MPEG_DIALOG, OGMRipMpegDialogClass))
#define OGMRIP_IS_MPEG_DIALOG(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), OGMRIP_TYPE_MPEG_DIALOG))
#define OGMRIP_IS_MPEG_DIALOG_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((klass), OGMRIP_TYPE_MPEG_DIALOG))

#define OGMRIP_GCONF_MPEG               "mpeg"
#define OGMRIP_GCONF_MPEG_FORMAT        "format"
#define OGMRIP_DEFAULT_MPEG_FORMAT      0

enum
{
  PROP_0,
  PROP_KEY
};

typedef struct _OGMRipMpegDialog      OGMRipMpegDialog;
typedef struct _OGMRipMpegDialogClass OGMRipMpegDialogClass;
typedef struct _OGMRipMpegOptions     OGMRipMpegOptions;

struct _OGMRipMpegDialog
{
  GtkDialog parent_instance;

  GtkWidget *format_combo;
};

struct _OGMRipMpegDialogClass
{
  GtkDialogClass parent_class;
};

struct _OGMRipMpegOptions
{
  gchar *name;
  gchar *key;
  gchar *type;
  gint  value[3];
};

static void ogmrip_mpeg_dialog_set_property  (GObject          *gobject,
                                              guint            property_id,
                                              const GValue     *value,
                                              GParamSpec       *pspec);
static void ogmrip_mpeg_dialog_set_gconf_key (OGMRipMpegDialog *dialog,
                                              const gchar      *key);

static void
ogmrip_mpeg_dialog_connect_combo (GtkWidget *combo, const gchar *dir, const gchar *key, gint def)
{
  GConfValue *value;
  gchar *real_key;

  if (dir)
    real_key = gconf_concat_dir_and_key (dir, key);
  else
    real_key = g_strdup (key);

  value = ogmrip_preferences_get (real_key);
  if (value)
    gconf_value_free (value);
  else
    ogmrip_preferences_set_int (real_key, def);

  ogmrip_preferences_connect_combo (combo, real_key);

  g_free (real_key);
}

G_DEFINE_TYPE (OGMRipMpegDialog, ogmrip_mpeg_dialog, GTK_TYPE_DIALOG)

static void
ogmrip_mpeg_dialog_class_init (OGMRipMpegDialogClass *klass)
{
  GObjectClass *gobject_class;

  gobject_class = (GObjectClass *) klass;
  gobject_class->set_property = ogmrip_mpeg_dialog_set_property;

  g_object_class_install_property (gobject_class, PROP_KEY, 
        g_param_spec_string ("key", "GConf key property", "Set the gconf key", 
           NULL, G_PARAM_WRITABLE));
}

static void
ogmrip_mpeg_dialog_init (OGMRipMpegDialog *dialog)
{
  GtkWidget *widget;
  GladeXML *xml;

  xml = glade_xml_new (OGMRIP_DATA_DIR "/" OGMRIP_GLADE_FILE, OGMRIP_GLADE_ROOT, NULL);
  if (!xml)
  {
    g_warning ("Could not find " OGMRIP_GLADE_FILE);
    return;
  }

  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
      NULL);
  gtk_window_set_title (GTK_WINDOW (dialog), _("MPEG Options"));
  gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_PREFERENCES);

  widget = glade_xml_get_widget (xml, OGMRIP_GLADE_ROOT);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), widget, TRUE, TRUE, 0);
  gtk_widget_show (widget);

  dialog->format_combo = glade_xml_get_widget (xml, "format-combo");

  g_object_unref (xml);
}

static void
ogmrip_mpeg_dialog_set_property (GObject *gobject, guint property_id, const GValue *value, GParamSpec *pspec)
{
  switch (property_id) 
  {
    case PROP_KEY:
      ogmrip_mpeg_dialog_set_gconf_key (OGMRIP_MPEG_DIALOG (gobject), g_value_get_string (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
      break;
  }
}

static void
ogmrip_mpeg_dialog_set_gconf_key (OGMRipMpegDialog *dialog, const gchar *key)
{
  ogmrip_mpeg_dialog_connect_combo (dialog->format_combo, key,
      OGMRIP_GCONF_MPEG "/" OGMRIP_GCONF_MPEG_FORMAT, OGMRIP_DEFAULT_MPEG_FORMAT);
}

static void
ogmrip_mpeg_set_options (OGMRipVideoCodec *codec)
{
#define GET_INT(key,def)  key, ogmrip_preferences_get_int  (OGMRIP_GCONF_MPEG "/" key, def)

  if (ogmrip_preferences_get_int ("video/preset", 0) == OGMRIP_VIDEO_PRESET_USER)
    g_object_set (codec,
        GET_INT  (OGMRIP_GCONF_MPEG_FORMAT, OGMRIP_DEFAULT_MPEG_FORMAT),
        NULL);

#undef GET_INT
}

static OGMRipVideoOptionsPlugin mpeg_options_plugin =
{
  NULL,
  G_TYPE_NONE,
  G_TYPE_NONE,
  NULL
};

OGMRipVideoOptionsPlugin *
ogmrip_init_options_plugin (void)
{
  mpeg_options_plugin.type = ogmrip_plugin_get_container_by_name ("mpeg");
  if (mpeg_options_plugin.type == G_TYPE_NONE)
    return NULL;

  mpeg_options_plugin.dialog = OGMRIP_TYPE_MPEG_DIALOG;

  mpeg_options_plugin.set_options = ogmrip_mpeg_set_options;

  return &mpeg_options_plugin;
}

